@@ -35,7 +35,7 @@ function sanity_check($string) {
 	if (preg_match("/\.\.\//", $string)) {
 		exit;
 	}
-	if (preg_match("/\//", $string)) {
+	if (preg_match("/^\//", $string)) {
 		exit;
 	}
 	if (preg_match("/;/", $string)) {
@@ -68,6 +68,49 @@ function is_song($str) {
 	return false;
 }
 
+function findsubdirs($dir) {
+	$dir = escapeshellcmd($dir);
+	$dirs = glob("$dir/");
+	foreach (glob("$dir/{.[^.]*,*}", GLOB_BRACE|GLOB_ONLYDIR) as $sub_dir){
+		$list = findsubdirs($sub_dir);
+		$dirs = array_merge($dirs, $list);
+	}
+	return $dirs;
+}
+
+function has_files($dir) {
+	foreach (scandir($dir) as $file) {
+		if (is_file("$dir/$file")) {
+			return true;
+		}
+	}
+	return false;
+}
+
+function split2($path) {
+	// Artist is everything before the first slash,
+	// Album is everything after
+	return preg_split("/\/+/", $path, 2);
+}
+
+function split3($path) {
+	// Artist is everything before the first slash,
+	// Song is everything after the last slash
+	// Album is everything in between
+	$results = preg_split("/\/+/", $path);
+	$artist = array_shift($results);
+	$song = array_pop($results);
+	$album = array_shift($results);
+	foreach ($results as $i) {
+		$album = "$album/$i";
+	}
+	return array($artist, $album, $song);
+}
+
+function urlencode_album($album) {
+	// Must allow "/" in album names, for nested albums
+	return preg_replace("/%2F/", "/", urlencode($album));
+}
 
 $top = sanity_check($_REQUEST["top"]);
 $middle = sanity_check($_REQUEST["middle"]);
@@ -143,7 +186,7 @@ function get_all_albums($search="") {
 	for ($i=0; $i<sizeof($artists); $i++) {
 		$a =  get_albums_by_artist($artists[$i]);
 		for ($j=0; $j<sizeof($a); $j++) {
-			list($artist, $album) = preg_split("/\//", $a[$j]);
+			list($artist, $album) = split2($a[$j]);
 			if (!$search || preg_match("/$search/i", $album)) {
 				array_push($albums, $a[$j]);
 			}
@@ -161,10 +204,11 @@ function get_all_songs($search="", $rating=0) {
 		$artists = get_all_artists();
 	}
 	$songs = array();
+	$images = array();
 	for ($i=0; $i<sizeof($artists); $i++) {
-		$s = get_songs_by_album($artists[$i]);
+		list($s, $images) = get_songs_by_album($artists[$i]);
 		for ($j=0; $j<sizeof($s); $j++) {
-			list($artist, $album, $song) = preg_split("/\//", $s[$j]);
+			list($artist, $album, $song) = split3($s[$j]);
 			if (!$search || preg_match("/$search/", $song)) {
 				if (!$rating || $ratings["$artist"] >= $rating) {
 					array_push($songs, $s[$j]);
@@ -174,10 +218,10 @@ function get_all_songs($search="", $rating=0) {
 	}
 	$albums = get_all_albums();
 	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = preg_split("/\//", $albums[$i]);
-		$s = get_songs_by_album($artist, $album);
+		list($artist, $album) = split2($albums[$i]);
+		list($s, $images) = get_songs_by_album($artist, $album);
 		for ($j=0; $j<sizeof($s); $j++) {
-			list($artist, $album, $song) = preg_split("/\//", $s[$j]);
+			list($artist, $album, $song) = split3($s[$j]);
 			if (!$search || preg_match("/$search/i", $song)) {
 				if (!$rating || $ratings["$artist"] >= $rating) {
 					array_push($songs, $s[$j]);
@@ -192,13 +236,15 @@ function get_all_songs($search="", $rating=0) {
 function get_albums_by_artist($artist) {
 	$albums = array();
 	if (is_dir("music/$artist")) {
-		if ($dir = opendir("music/$artist")) {
-			while (($album = readdir($dir)) !== false){
-				if (is_dir("music/$artist/$album") && visible($artist) && visible($album)) {
-					array_push($albums, "$artist/$album");
-				}
-			}
-			closedir($dir);
+		$albums = findsubdirs("music/$artist");
+	}
+	array_shift($albums);
+	for ($i=0; $i<sizeof($albums); $i++) {
+		if (! has_files($albums[$i])) {
+			$albums[$i] = "";
+		} else {
+			$albums[$i] = preg_replace("/^music\//", "", $albums[$i]);
+			$albums[$i] = preg_replace("/\/$/", "", $albums[$i]);
 		}
 	}
 	sort($albums);
@@ -229,10 +275,7 @@ function get_rating($artist) {
 	}
 	$str = "";
 	for ($i=1; $i<=$r; $i++) {
-		$str .= "<a href=?rating=$i&artist=" . urlencode($artist) . "><img border=0 src=star_on.png></a> ";
-	}
-	for ($i=$r+1; $i<=5; $i++) {
-		$str .= "<a href=?rating=$i&artist=" . urlencode($artist) . "><img border=0 src=star_off.png></a> ";
+		$str .= "<a href=?rating=$i&artist=" . urlencode($artist) . "><img border=0 src=silk/star.png></a> ";
 	}
 	return $str;
 }
@@ -257,25 +300,31 @@ function set_rating($artist, $rating) {
 
 function get_songs_by_album($artist, $album="") {
 	$songs = array();
+	$images = array();
 	if (is_dir("music/$artist/$album")) {
 		if ($dir = opendir("music/$artist/$album")) {
-			while (($song = readdir($dir)) !== false){
-				if (is_song("music/$artist/$album/$song") && visible($artist) && visible($album) && visible($song)) {
-					array_push($songs, "$artist/$album/$song");
+			while (($file = readdir($dir)) !== false){
+				if (visible($artist) && visible($album) && visible($file)) {
+					if (is_song("music/$artist/$album/$file")) {
+						array_push($songs, "$artist/$album/$file");
+					} elseif (exif_imagetype("music/$artist/$album/$file")) {
+						array_push($images, "music/$artist/$album/$file");
+					}
 				}
 			}
 			closedir($dir);
 		}
 	}
 	sort($songs);
-	return $songs;
+	return array($songs, $images);
 }
 
 function get_songs_by_artist($artist) {
 	$songs = array();
-	$s = get_songs_by_album($artist);
+	$images = array();
+	list($s, $images) = get_songs_by_album($artist);
 	for ($i=0; $i<sizeof($s); $i++) {
-		list($artist, $album, $song) = preg_split("/\//", $s[$i]);
+		list($artist, $album, $song) = split3($s[$i]);
 		if (!$search || preg_match("/$search/", $song)) {
 			if (!$rating || $ratings["$artist"] >= $rating) {
 				array_push($songs, $s[$i]);
@@ -284,10 +333,10 @@ function get_songs_by_artist($artist) {
 	}
 	$albums = get_albums_by_artist($artist);
 	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = preg_split("/\//", $albums[$i]);
-		$s = get_songs_by_album($artist, $album);
+		list($artist, $album) = split2($albums[$i]);
+		list($s, $images) = get_songs_by_album($artist, $album);
 		for ($j=0; $j<sizeof($s); $j++) {
-			list($artist, $album, $song) = preg_split("/\//", $s[$j]);
+			list($artist, $album, $song) = split3($s[$j]);
 			if (!$search || preg_match("/$search/i", $song)) {
 				if (!$rating || $ratings["$artist"] >= $rating) {
 					array_push($songs, $s[$j]);
@@ -300,7 +349,9 @@ function get_songs_by_artist($artist) {
 }
 
 function get_size_of_album($artist, $album) {
-	$songs = get_songs_by_album($artist, $album);
+	$songs = array();
+	$images = array();
+	list($songs, $images) = get_songs_by_album($artist, $album);
 	$size = 0;
 	for ($i=0; $i<sizeof($songs); $i++) {
 		$size += filesize("music/$songs[$i]");
@@ -322,7 +373,7 @@ function get_counts() {
 	$artists = @file("/var/lib/musica/artists.count");
 	$albums = @file("/var/lib/musica/albums.count");
 	$songs = @file("/var/lib/musica/songs.count");
-	$counts = array($artists[0], $albums[0], $songs[0]);
+	$counts = array(rtrim($artists[0]), rtrim($albums[0]), rtrim($songs[0]));
 	return $counts;
 }
 
@@ -336,11 +387,6 @@ function get_artist_ratings() {
 	return $ratings;
 }
 
-function filename($file) {
-	$parts = preg_split("/\/+/", $file);
-	return array_pop($parts);
-}
-
 /*******************/
 /* print functions */
 /*******************/
@@ -348,14 +394,14 @@ function filename($file) {
 function print_artist($artist) {
 	if (is_dir("music/$artist") && visible($artist)) {
 		$href = "?artist=" . urlencode($artist);
-		print("<li><a href=\"$href\" target=_albums><img src=group.png>&nbsp;$artist</a></li>\n");
+		print("<li><a href=\"$href\" target=_albums><img src=silk/group.png>&nbsp;$artist</a></li>\n");
 	}
 }
 
 function print_album($artist, $album) {
-	if (visible($artist) && visible($album)) {
-		$href = "?artist=" . urlencode($artist) . "&album=" . urlencode($album);
-		print("<a href=$href target=_songs><img border=0 src=cd.png>&nbsp;$album</a><br>");
+	if (visible($artist) && visible($album) && isset($album)) {
+		$href = "?artist=" . urlencode($artist) . "&album=" . urlencode_album($album);
+		print("<li><a href=$href target=_songs><img border=0 src=silk/cd.png>&nbsp;$album</a><br>");
 	}
 }
 
@@ -363,14 +409,15 @@ function print_song($artist, $album, $song) {
 	global $PREAMBLE, $JPLAYER_LIST, $JPLAYER_OGG;
 	print("<noscript>");
 	if (is_song($song) && visible($artist) && visible($album) && visible($song)) {
-		$href = $PREAMBLE . urlencode($artist) . "/" . urlencode($album) . "/" . urlencode("$song");
+		$href = $PREAMBLE . urlencode($artist) . "/" . urlencode_album($album) . "/" . urlencode("$song");
 		$href = $line = preg_replace("/\+/", "%20", $href);
 		$parts = pathinfo($song);
-		if ($ext == "ogg" || $ext == "OGG") {
+		if (preg_match("/^og/i", $parts['extension'])) {
 			$JPLAYER_OGG = "true";
+			$parts['extension'] = "ogg";
 		}
-		print("<a href=\"$href\"><img src=disk.png border=0></a><a href=\"?playlist=1&artist=$artist&album=$album&song=$song\">");
-		print("<img src=music.png border=0>&nbsp;" . $parts["filename"] . "</a><br>");
+		print("<a href=\"$href\"><img src=silk/disk.png border=0></a><a href=\"?playlist=1&artist=$artist&album=$album&song=$song\">");
+		print("<img src=silk/music.png border=0>&nbsp;" . $parts["filename"] . "</a><br>");
 		$JPLAYER_LIST .= "{name:\"" . $parts["filename"] . "\"," . $parts["extension"] . ":\"$href\"},\n";
 	}
 	print("</noscript>");
@@ -393,10 +440,7 @@ function print_artists($search="") {
 	for ($i=5; $i>=1; $i--) {
 		print("<li><a href=?playlist=$i target=_songs>");
 		for ($j=1; $j<=$i; $j++) {
-			print("<img border=0 src=star_on.png>");
-		}
-		for ($j=$i+1; $j<=5; $j++) {
-			print("<img border=0 src=star_off.png>");
+			print("<img border=0 src=silk/star.png>");
 		}
 		print("</a></li>");
 	}
@@ -409,16 +453,15 @@ function print_albums_by_artist($artist) {
 		set_rating($artist, $rating);
 	}
 	$albums = get_albums_by_artist($artist);
-	print("<p align=center><img src=group.png>&nbsp;<big><b>$artist</b></big><br>");
+	print("<p align=center><img src=silk/group.png>&nbsp;<big><b>$artist</b></big><br>");
 	$wiki = preg_replace("/\s*\(.*/", "", $artist);
 	$wiki = urlencode($wiki);
 	$wiki = preg_replace("/\+/", "_", $wiki);
-	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=book_open.png border=0> Wikipedia</a><br>");
+	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=silk/book_open.png border=0> Wikipedia</a><br>");
 	print( get_rating($artist) . "</p><ol>");
 	$loadmisc = 0;
 	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = preg_split("/\//", $albums[$i]);
-		print("<li>");
+		list($artist, $album) = preg_split("/\//", $albums[$i], 2);
 		print_album($artist, $album);
 	}
 	if (!$loadmisc) {
@@ -432,7 +475,7 @@ function print_albums_by_search($search) {
 	print("<center><big><b>Matching Artists or Albums</b></big></center><ol>");
 	print("<ol>");
 	for ($i=0; $i<sizeof($albums); $i++) {
-		list($artist, $album) = preg_split("/\//", $albums[$i]);
+		list($artist, $album) = split2($albums[$i]);
 		print_artist($artist);
 		print_album($artist, $album);
 	}
@@ -441,14 +484,15 @@ function print_albums_by_search($search) {
 
 function print_songs_by_album($artist, $album) {
 	global $JPLAYER_LIST, $JPLAYER_OGG;
-	print("<p align=center><img src=group.png>&nbsp;<b><big>$artist</b></big><br><img src=cd.png>&nbsp;<b>$album</b><br>\n");
+	print("<p align=center><img src=silk/group.png>&nbsp;<b><big>$artist</b></big><br><img src=silk/cd.png>&nbsp;<b>$album</b><br>\n");
 	$wiki = preg_replace("/\s*\(.*/", "", $album);
 	$wiki = urlencode($wiki);
 	$wiki = preg_replace("/\+/", "_", $wiki);
-	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=book_open.png border=0> Wikipedia</a><br></p>");
-	$songs = get_songs_by_album($artist, $album);
+	print("<a target=_new href=http://en.wikipedia.org/wiki/$wiki><img width=16 heigh=16 src=silk/book_open.png border=0> Wikipedia</a><br></p>");
+	list($songs, $images) = get_songs_by_album($artist, $album);
+	print_album_cover($images, "$artist/$album");
 	for ($i=0; $i<sizeof($songs); $i++) {
-		list($artist, $album, $song) = preg_split("/\//", $songs[$i]);
+		list($artist, $album, $song) = split3($songs[$i]);
 		print_song($artist, $album, $song);
 	}
 	if ($JPLAYER_LIST != "") {
@@ -456,23 +500,27 @@ function print_songs_by_album($artist, $album) {
 	}
 	$size = get_size_of_album($artist, $album);
         print("<p align=right>
-<a href=?playlist=1&artist=" . urlencode($artist) . "&album=" . urlencode($album) . "><img src=control_play_blue.png border=0>&nbsp;play album</a><br>
-<a href=?download_album=1&artist=" . urlencode($artist) . "&album=" . urlencode($album) . "><img src=disk.png border=0>&nbsp;download</a> $size
+<a target=_new href=" . $_SERVER["REQUEST_URI"] . "&popout=1><img src=silk/application_double.png border=0>&nbsp;play in a new window</a><br>
+<a href=?playlist=1&artist=" . urlencode($artist) . "&album=" . urlencode_album($album) . "><img src=silk/control_play_blue.png border=0>&nbsp;play album</a><br>
+<a href=?download_album=1&artist=" . urlencode($artist) . "&album=" . urlencode_album($album) . "><img src=silk/disk.png border=0>&nbsp;download</a> $size
 	");
 }
 
 function print_misc_songs_by_artist($artist) {
-	global $JPLAYER_LIST, $JPLAYER_OGG;
-	print("<center><img src=group.png>&nbsp;<big><b>$artist</b></big><br><img src=music.png>&nbsp;<b>Miscellaneous</b></center><br>");
-	$songs = get_songs_by_album($artist);
+	global $JPLAYER_LIST, $JPLAYER_OGG, $_SERVER;
+	print("<center><img src=silk/group.png>&nbsp;<big><b>$artist</b></big><br><img src=silk/music.png>&nbsp;<b>Miscellaneous</b></center><br>");
+	list($songs, $images) = get_songs_by_album($artist);
+	print_album_cover($images, $artist);
 	for ($i=0; $i<sizeof($songs); $i++) {
-		list($artist, $album, $song) = preg_split("/\//", $songs[$i]);
+		list($artist, $album, $song) = split3($songs[$i]);
 		print_song($artist, $album, $song);
 	}
 	if ($JPLAYER_LIST != "") {
 		include("jplayer.php");
 	}
-	print("<p align=right><a href=?playlist=1&artist=" . urlencode($artist) . "><img src=control_play_blue.png border=0>&nbsp;play all by artist</a></p> ");
+	print("<p align=right>
+<a target=_new href=" . $_SERVER["REQUEST_URI"] . "&popout=1><img src=silk/application_double.png border=0>&nbsp;play in a new window</a><br>
+<a href=?playlist=1&artist=" . urlencode($artist) . "><img src=silk/control_play_blue.png border=0>&nbsp;play all by artist</a></p> ");
 }
 
 function print_songs_by_search($search) {
@@ -480,7 +528,7 @@ function print_songs_by_search($search) {
         print("<ol>");
 	$songs = get_all_songs($search);
 	for ($i=0; $i<sizeof($songs); $i++) {
-		list($artist, $album, $song) = preg_split("/\//", $songs[$i]);
+		list($artist, $album, $song) = split3($songs[$i]);
 		//print_artist($artist);
 		//print_album($artist, $album);
 		print_song($artist, $album, $song);
@@ -492,6 +540,49 @@ function print_songs_by_search($search) {
 	print("</ol>");
 }
 
+function print_album_cover($images, $dir) {
+	$width = 200;
+	$js = "src = [";
+	for ($i=0; $i<sizeof($images); $i++) {
+		if ($i > 0) {
+			$js .= ",";
+		}
+		$js .= "'" . $images[$i] . "'";
+	}
+	$js .= "]";
+	if ($i>= 1) {
+		# From http://www.javascriptkit.com/script/script2/incrementslide.shtml
+		print("
+<script>
+" . $js . "
+pics=[]; i=0;
+function switch_pic() {
+  var n=(i+1)%src.length;
+  if (pics[n] && (pics[n].complete || pics[n].complete==null)) {
+    document['albumart'].src = pics[i=n].src;
+  }
+  pics[n=(i+1)%src.length] = new Image;
+  pics[n].src = src[n];
+  setTimeout('switch_pic()',4000);
+} onload = function(){
+  if (document.images)
+    switch_pic();
+}
+</script>");
+		$img = "<center><img name=albumart width=$width src='$images[0]'></center><br>";
+	} else {
+		$tmp = "/var/lib/musica/tmp";
+		$output = `eyeD3 -i $tmp "/usr/share/musica/music/$dir" 2>&1`;
+		$output = preg_replace("/.*Writing /", "", $output);
+		list($filename, $rest) = preg_split("/\.\.\./", $output);
+		if (file_exists($filename)) {
+			$img = "<center><img width=$width src=data:image/gif;base64," . base64_encode(file_get_contents("$filename")) . $img . "></center><br>";
+			unlink($filename);
+		}
+	}
+	print($img);
+}
+
 
 /**********/
 /* Frames */
@@ -556,16 +647,16 @@ if ($top) {
 <table border=0 width=100% cellpadding=0 cellspacing=0>
   <tr align=center>
     <td width=33%>
-      <img src=group.png>&nbsp;<b>Artists ($artists)</b><br>
+      <img src=silk/group.png>&nbsp;<b>Artists ($artists)</b><br>
     </td>
     <td width=33%>
-      <img src=cd.png>&nbsp;<b>Albums ($albums)</b>
+      <img src=silk/cd.png>&nbsp;<b>Albums ($albums)</b>
     </td>
     <td width=33%>
-      <img src=music.png>&nbsp;<b>Songs ($songs)</b>
+      <img src=silk/music.png>&nbsp;<b>Songs ($songs)</b>
     </td>
     <td width=1%>
-      <b><a href=?about=1 target=_songs title='About Musica'>?</a></b>
+      <a href=?about=1 target=_songs title='About Musica'><img src=silk/help.png></a>
     </td>
   </tr>
   <tr align=center>
@@ -601,7 +692,7 @@ if ($about) {
 
 <a href=https://launchpad.net/musica target=_top>Musica</a> is Free Software under the <a href=http://www.gnu.org/licenses/agpl-3.0.txt target=_top>GNU AGPL 3</a>, Copyright &copy;2000-2010 <a href=http://blog.dustinkirkland.com>Dustin Kirkland</a>.<br><br>
 
-<a href=http://www.famfamfam.com/lab/icons/silk/ target=_top>FamFamFam Silk Icons</a> are used under the <a href=http://creativecommons.org/licenses/by/3.0/ target=_top>CCA3.0</a> license.<br><br>
+<a href=http://www.famfamfam.com/lab/silk/silk/ target=_top>FamFamFam Silk Icons</a> are used under the <a href=http://creativecommons.org/licenses/by/3.0/ target=_top>CCA3.0</a> license.<br><br>
 
 <a href=http://www.happyworm.com/jquery/jplayer/ target=_top>jPlayer audio player plugin</a> is developed by <a href=http://www.happyworm.com/ target=_top>Happyworm</a>, and is Free, Open Source and dual licensed under the <a href=http://www.opensource.org/licenses/mit-license.php target=_top>MIT</a> and <a href=http://www.gnu.org/copyleft/gpl.html target=_top>GPL</a> licenses.<br><br>
 	");
@@ -688,7 +779,7 @@ if ($playlist) {
 		array_push($songs, "$artist/$album/$song");
 		$filename = urlencode($artist) . "_" . urlencode($album) . "_" . urlencode($song) . ".m3u";
 	} elseif ($album) {
-		$songs = get_songs_by_album($artist, $album);
+		list($songs, $images) = get_songs_by_album($artist, $album);
 		$filename = urlencode($artist) . "_" . urlencode($album) . ".m3u";
 	} elseif ($artist) {
 		$songs = get_songs_by_artist($artist);
